var https = require("https");

exports.handler = function(event, context) {
/* Define date information used to create expiration date for test CC */
var date = new Date();
var currentYear = date.getFullYear().toString().substr(2,2);
var currentMonth = date.getMonth() + 1;
if (currentMonth < 10) { currentMonth = "0" + currentMonth; }

/* Define Authorize.Net Connection */
var authnetValidateCert = "false";	/* In production this should be set to true to validate 3rd party certificate */
var authnetHost = "";
var authnetHostBeta = "apitest.authorize.net";
var authnetHostLive = "api.authorize.net";
var authnetPort = 443;
var authnetPath = "/xml/v1/request.api";
var authnetMethod = "POST";
var authnetContentType = "text/xml";
var authnetResponseData = [];

/* Define Authorize.Net Gateway credentials */
var gateway_login = "gatewaylogindetails";
var gateway_key = "gatewaykeydetails";

/* Define user purchase details */
var isTestTransaction = 1;  /* 1 = test transaction, 0 = not test transaction */
var amount = "4.71";
var ccnum = "6011000000000012"; /* Test CC 5424000000000015 */
var cvv = "330";    /* Test CVV 999 */
var expdate = currentMonth + currentYear; /* Today's date in MMYY format for testing */

/* Define request body */
/*
   NOTE: As of 1/2016 AWS/Lambda does not support XML out of the box, as it were.  Without adding complexity overhead by rummaging around with
   other 3rd party node.js libraries (such as xml2js) because node.js ran from XML, and avoid needing to incur additional overhead (making a zip
   file for AWS/Lambda which includes your code and the 3rd party library - thus losing the ability to edit trivial lambda functions in the inline
   function editor page).  Ergo, handle XML the "old school" way.
*/
var postxmlPayload = "<createTransactionRequest xmlns=\"AnetApi/xml/v1/schema/AnetApiSchema.xsd\">";
postxmlPayload = postxmlPayload + "  <merchantAuthentication>";
postxmlPayload = postxmlPayload + "    <name>" + gateway_login + "</name>";
postxmlPayload = postxmlPayload + "    <transactionKey>" + gateway_key + "</transactionKey>";
postxmlPayload = postxmlPayload + " </merchantAuthentication>";
postxmlPayload = postxmlPayload + " <transactionRequest>";
postxmlPayload = postxmlPayload + "    <transactionType>authCaptureTransaction</transactionType>";
postxmlPayload = postxmlPayload + "    <amount>" + amount + "</amount>";
postxmlPayload = postxmlPayload + "    <payment>";
postxmlPayload = postxmlPayload + "        <creditCard>";
postxmlPayload = postxmlPayload + "          <cardNumber>" + ccnum + "</cardNumber>";
postxmlPayload = postxmlPayload + "          <expirationDate>" + expdate + "</expirationDate>";
postxmlPayload = postxmlPayload + "          <cardCode>" + cvv + "</cardCode>";
postxmlPayload = postxmlPayload + "        </creditCard>";
postxmlPayload = postxmlPayload + "    </payment>";
postxmlPayload = postxmlPayload + "  </transactionRequest>";
postxmlPayload = postxmlPayload + "</createTransactionRequest>";

/* Define which merchant URL to use */
if (isTestTransaction === 0) {
	authnetHost = authnetHostLive;
}
else {
	authnetHost = authnetHostBeta;
}

/* Define request parameters */
var curlOptions = {
	host: authnetHost,
	port: authnetPort,
	path: authnetPath,
	rejectUnauthorized: authnetValidateCert,
	method: authnetMethod,
	headers: {
		'Content-Type': authnetContentType,
		'Content-Length': Buffer.byteLength(postxmlPayload)
	}
};

/* Perform request */
var authnetRequest = https.request(curlOptions, function(result) {
	var responseData = "";
	/* Error */
	result.on('error', function(e) {
		context.fail('error: ' + e.message);
	});

	/* Collect Response */
	result.on('data', function(chunk)  {
		responseData += chunk;
	});

	/* Req / Res End. */
	result.on('end', function() {
	    /* Parse XML response data the "old school" way.  At least there's REGEX nowadays. */
	    var cleanedResponse = responseData.replace(/\t/g, "").replace(/\r\n/g, "").replace(/\n/g, "").replace(/\r/g, "").replace(/\f/g, "");
	    /* Extract Specific Values */
	    var transactionStatus = cleanedResponse.match(/(.*)<transactionResponse\>(.*)<responseCode\>(.*)<\/responseCode\>(.*)<\/transactionResponse\>(.*)/)[3];
        var transactionId = cleanedResponse.match(/(.*)<transactionResponse\>(.*)<transId\>(.*)<\/transId\>(.*)<\/transactionResponse\>(.*)/)[3];
        var transactionText = cleanedResponse.match(/(.*)<transactionResponse\>(.*)<messages\>(.*)<message\>(.*)<description\>(.*)<\/description\>(.*)<\/transactionResponse\>(.*)/)[5];
	    /* Generate Response */
		context.done(
			null,
			{
			"transactionStatus": transactionStatus,
			"transactionText": transactionText,
			"transactionId": transactionId
			}
		);
	});
});

/* Call the https request to run the transaction and continue with other logic tasks. */
authnetRequest.write(postxmlPayload);
authnetRequest.end(); /* Required "end" statement by Node.js standard */

};
